package com.stormtales.notevault.data.dao;

import androidx.lifecycle.LiveData;
import androidx.room.*;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.sync.SyncStatus;

import java.util.List;

@Dao
public interface SongDao {
    @Insert
    long insert(Song song);

    @Query("SELECT * FROM song WHERE syncStatus != 1")
    List<Song> getAllSongs();

    @Update
    void update(Song song);

    @Insert
    void insert(List<NoteSheet> noteSheets);

    @Delete
    void deleteNoteSheets(List<NoteSheet> noteSheets);

    @Query("SELECT * FROM NoteSheet WHERE songID = :songID")
    List<NoteSheet> getNoteSheetsBySong(int songID);

    @Query("SELECT localFileName FROM NoteSheet WHERE songID = :songID")
    List<String> getNoteSheetFilesBySongID(int songID);

    @Query("SELECT * FROM Song WHERE syncStatus = :syncStatus")
    List<Song> getSongsBySyncStatus(SyncStatus syncStatus);

    @Update
    void updateSongs(List<Song> songs);

    @Query("SELECT * FROM Song WHERE localID IN (:localIDs)")
    List<Song> getSongsByLocalIDs(List<Integer> localIDs);

    @Query("SELECT * FROM Song WHERE serverID IN (:serverIDs)")
    List<Song> getSongsByServerIDs(List<String> serverIDs);

    @Query("DELETE FROM Song WHERE serverID IN (:serverIDs)")
    void deleteSongsByServerIDs(List<String> serverIDs);

    @Query("DELETE FROM SONG WHERE localID IN (:localIDs)")
    void deleteSongsByLocalIDs(List<Integer> localIDs);

    @Query("SELECT * FROM NoteSheet WHERE songID IN (:localSongIDs)")
    List<NoteSheet> getNoteSheetsBySongIDs(List<Integer> localSongIDs);

    @Query("SELECT * FROM NoteSheet WHERE songID IN (:songIDs)")
    List<NoteSheet> getNoteSheetsByLocalFiles(List<Integer> songIDs);

    @Update
    void updateNoteSheets(List<NoteSheet> uploadedNoteSheets);

    @Query("SELECT localID FROM Song WHERE serverID IN (:serverIDs)")
    List<Integer> getLocalSongIDsByServerIDs(List<String> serverIDs);

    @Query("SELECT * FROM NoteSheet WHERE serverFileName IN (:serverFileNames)")
    List<NoteSheet> getNoteSheetsByServerFileNames(List<String> serverFileNames);

    @Query("SELECT * FROM NoteSheet WHERE syncStatus = :status")
    List<NoteSheet> getNoteSheetsBySyncStatus(SyncStatus status);

    @Query("SELECT * FROM NoteSheet WHERE songID IN (:localSongIDs)")
    List<NoteSheet> getNoteSheetFilesBySongIDs(List<Integer> localSongIDs);

    @Insert
    void insertSongs(List<Song> onlyRemoteExistingSongs);

    @Delete
    void deleteSong(Song song);

    @Query("DELETE FROM NoteSheet WHERE songID = :localID")
    void deleteNoteSheetsByLocalSongID(int localID);

    @Update
    void updateNoteSheet(NoteSheet noteSheet);

    @Query("SELECT * FROM NoteSheet WHERE serverFileName =:serverFileName")
    NoteSheet getNoteSheetByServerFileName(String serverFileName);
}
