package com.stormtales.notevault.ui.gallery;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;
import com.stormtales.notevault.data.repositories.SongRepository;
import com.stormtales.notevault.network.sync.SongSyncModule;

public class GalleryViewModel extends ViewModel {

    private final MutableLiveData<Boolean> isCreatedSongSyncing;
    private final MutableLiveData<Boolean> isModifiedSongSyncing;
    private final MutableLiveData<Boolean> isDeletedSongSyncing;
    private final MutableLiveData<Boolean> isFetchingActive;
    private SongSyncModule songSyncModule;

    public GalleryViewModel() {

        isCreatedSongSyncing = new MutableLiveData<>();
        isCreatedSongSyncing.setValue(false);

        isModifiedSongSyncing = new MutableLiveData<>();
        isModifiedSongSyncing.setValue(false);

        isDeletedSongSyncing = new MutableLiveData<>();
        isDeletedSongSyncing.setValue(false);

        isFetchingActive = new MutableLiveData<>();
        isFetchingActive.setValue(false);
    }

    public MutableLiveData<Boolean> getIsCreatedSongSyncing() {
        return isCreatedSongSyncing;
    }

    public void startCreateSongSyncing() {
        if(this.songSyncModule == null) throw new RuntimeException("SongSyncModule is not initialized");
        this.isCreatedSongSyncing.setValue(true);

        this.songSyncModule.syncCreatedSongs();
    }

    public void finishCreateSongSyncing() {
        this.isCreatedSongSyncing.setValue(false);
    }

    public void setSongSyncModule(SongSyncModule songSyncModule) {
        this.songSyncModule = songSyncModule;
    }

    public void startModifiedSongSyncinc() {
        if(this.songSyncModule == null) throw new RuntimeException("SongSyncModule is not initialized");
        this.isModifiedSongSyncing.setValue(true);
        this.songSyncModule.syncModifiedSongs();
    }

    public MutableLiveData<Boolean> getIsModifiedSongSyncing() {
        return isModifiedSongSyncing;
    }

    public void finishModifiedSongSyncinc() {
        this.isModifiedSongSyncing.setValue(false);
    }

    public MutableLiveData<Boolean> getIsDeletedSongSyncing() {
        return isDeletedSongSyncing;
    }

    public void startDeletedSongSyncing() {
        if(this.songSyncModule == null) throw new RuntimeException("SongSyncModule is not initialized");
        this.isDeletedSongSyncing.setValue(true);
        this.songSyncModule.syncDeletedSongs();
    }

    public void finishDeleteSongSyncinc() {
        this.isDeletedSongSyncing.setValue(false);
    }

    public void startFetchingActive() {
        if(this.songSyncModule == null) throw new RuntimeException("SongSyncModule is not initialized");
        this.isFetchingActive.setValue(true);
        this.songSyncModule.fetchRemoteModifiedSongs();
    }

    public void finishFetching() {
        this.isFetchingActive.setValue(false);
    }

    public MutableLiveData<Boolean> getIsFetchingActive() {
        return isFetchingActive;
    }
}